@@ -29,7 +29,7 @@ class YoutubeDLDownloader(DownloaderBase):
         }
 
         self.ytdl_instance = None
-        self.forward_cookies = self.config("forward-cookies", False)
+        self.forward_cookies = self.config("forward-cookies", True)
         self.progress = self.config("progress", 3.0)
         self.outtmpl = self.config("outtmpl")
 
@@ -48,11 +48,20 @@ class YoutubeDLDownloader(DownloaderBase):
                     self.log.debug("", exc_info=exc)
                     self.download = lambda u, p: False
                     return False
+
+                try:
+                    ytdl_version = module.version.__version__
+                except Exception:
+                    ytdl_version = ""
+                self.log.debug("Using %s version %s", module, ytdl_version)
+
                 self.ytdl_instance = ytdl_instance = ytdl.construct_YoutubeDL(
                     module, self, self.ytdl_opts)
                 if self.outtmpl == "default":
                     self.outtmpl = module.DEFAULT_OUTTMPL
             if self.forward_cookies:
+                self.log.debug("Forwarding cookies to %s",
+                               ytdl_instance.__module__)
                 set_cookie = ytdl_instance.cookiejar.set_cookie
                 for cookie in self.session.cookies:
                     set_cookie(cookie)
