@@ -20,6 +20,7 @@ class DanbooruExtractor(BaseExtractor):
     page_limit = 1000
     page_start = None
     per_page = 200
+    useragent = util.USERAGENT
     request_interval = (0.5, 1.5)
 
     def _init(self):
@@ -31,7 +32,7 @@ class DanbooruExtractor(BaseExtractor):
         if isinstance(threshold, int):
             self.threshold = 1 if threshold < 1 else threshold
         else:
-            self.threshold = self.per_page
+            self.threshold = self.per_page - 20
 
         username, api_key = self._get_auth_info()
         if username:
@@ -46,8 +47,8 @@ class DanbooruExtractor(BaseExtractor):
         return pages * self.per_page
 
     def items(self):
-        self.session.headers["User-Agent"] = util.USERAGENT
-
+        # 'includes' initialization must be done here and not in '_init()'
+        # or it'll cause an exception with e621 when 'metadata' is enabled
         includes = self.config("metadata")
         if includes:
             if isinstance(includes, (list, tuple)):
@@ -93,7 +94,9 @@ class DanbooruExtractor(BaseExtractor):
 
             if post["extension"] == "zip":
                 if self.ugoira:
-                    post["frames"] = self._ugoira_frames(post)
+                    post["_ugoira_original"] = False
+                    post["_ugoira_frame_data"] = post["frames"] = \
+                        self._ugoira_frames(post)
                     post["_http_adjust_extension"] = False
                 else:
                     url = post["large_file_url"]
@@ -106,6 +109,13 @@ class DanbooruExtractor(BaseExtractor):
             yield Message.Directory, post
             yield Message.Url, url, post
 
+    def items_artists(self):
+        for artist in self.artists():
+            artist["_extractor"] = DanbooruTagExtractor
+            url = "{}/posts?tags={}".format(
+                self.root, text.quote(artist["name"]))
+            yield Message.Queue, url, artist
+
     def metadata(self):
         return ()
 
@@ -195,12 +205,8 @@ class DanbooruTagExtractor(DanbooruExtractor):
     pattern = BASE_PATTERN + r"/posts\?(?:[^&#]*&)*tags=([^&#]*)"
     example = "https://danbooru.donmai.us/posts?tags=TAG"
 
-    def __init__(self, match):
-        DanbooruExtractor.__init__(self, match)
-        tags = match.group(match.lastindex)
-        self.tags = text.unquote(tags.replace("+", " "))
-
     def metadata(self):
+        self.tags = text.unquote(self.groups[-1].replace("+", " "))
         return {"search_tags": self.tags}
 
     def posts(self):
@@ -214,7 +220,7 @@ class DanbooruTagExtractor(DanbooruExtractor):
                 else:
                     prefix = None
             elif tag.startswith(
-                    ("id:", "md5", "ordfav:", "ordfavgroup:", "ordpool:")):
+                    ("id:", "md5:", "ordfav:", "ordfavgroup:", "ordpool:")):
                 prefix = None
                 break
 
@@ -225,15 +231,13 @@ class DanbooruPoolExtractor(DanbooruExtractor):
     """Extractor for posts from danbooru pools"""
     subcategory = "pool"
     directory_fmt = ("{category}", "pool", "{pool[id]} {pool[name]}")
+    filename_fmt = "{num:>04}_{id}_{filename}.{extension}"
     archive_fmt = "p_{pool[id]}_{id}"
     pattern = BASE_PATTERN + r"/pool(?:s|/show)/(\d+)"
     example = "https://danbooru.donmai.us/pools/12345"
 
-    def __init__(self, match):
-        DanbooruExtractor.__init__(self, match)
-        self.pool_id = match.group(match.lastindex)
-
     def metadata(self):
+        self.pool_id = self.groups[-1]
         url = "{}/pools/{}.json".format(self.root, self.pool_id)
         pool = self.request(url).json()
         pool["name"] = pool["name"].replace("_", " ")
@@ -241,8 +245,42 @@ class DanbooruPoolExtractor(DanbooruExtractor):
         return {"pool": pool}
 
     def posts(self):
-        params = {"tags": "pool:" + self.pool_id}
-        return self._pagination("/posts.json", params, "b")
+        reverse = prefix = None
+
+        order = self.config("order-posts")
+        if not order or order in ("asc", "pool", "pool_asc", "asc_pool"):
+            params = {"tags": "ordpool:" + self.pool_id}
+        elif order in ("id", "desc_id", "id_desc"):
+            params = {"tags": "pool:" + self.pool_id}
+            prefix = "b"
+        elif order in ("desc", "desc_pool", "pool_desc"):
+            params = {"tags": "ordpool:" + self.pool_id}
+            reverse = True
+        elif order in ("asc_id", "id_asc"):
+            params = {"tags": "pool:" + self.pool_id}
+            reverse = True
+
+        posts = self._pagination("/posts.json", params, prefix)
+        if reverse:
+            return self._enumerate_posts_reverse(posts)
+        else:
+            return self._enumerate_posts(posts)
+
+    def _enumerate_posts(self, posts):
+        pid_to_num = {pid: num+1 for num, pid in enumerate(self.post_ids)}
+        for post in posts:
+            post["num"] = pid_to_num[post["id"]]
+            yield post
+
+    def _enumerate_posts_reverse(self, posts):
+        self.log.info("Collecting posts of pool %s", self.pool_id)
+        posts = list(posts)
+        posts.reverse()
+
+        pid_to_num = {pid: num+1 for num, pid in enumerate(self.post_ids)}
+        for post in posts:
+            post["num"] = pid_to_num[post["id"]]
+        return posts
 
 
 class DanbooruPostExtractor(DanbooruExtractor):
@@ -252,12 +290,8 @@ class DanbooruPostExtractor(DanbooruExtractor):
     pattern = BASE_PATTERN + r"/post(?:s|/show)/(\d+)"
     example = "https://danbooru.donmai.us/posts/12345"
 
-    def __init__(self, match):
-        DanbooruExtractor.__init__(self, match)
-        self.post_id = match.group(match.lastindex)
-
     def posts(self):
-        url = "{}/posts/{}.json".format(self.root, self.post_id)
+        url = "{}/posts/{}.json".format(self.root, self.groups[-1])
         post = self.request(url).json()
         if self.includes:
             params = {"only": self.includes}
@@ -273,12 +307,8 @@ class DanbooruPopularExtractor(DanbooruExtractor):
     pattern = BASE_PATTERN + r"/(?:explore/posts/)?popular(?:\?([^#]*))?"
     example = "https://danbooru.donmai.us/explore/posts/popular"
 
-    def __init__(self, match):
-        DanbooruExtractor.__init__(self, match)
-        self.params = match.group(match.lastindex)
-
     def metadata(self):
-        self.params = params = text.parse_query(self.params)
+        self.params = params = text.parse_query(self.groups[-1])
         scale = params.get("scale", "day")
         date = params.get("date") or datetime.date.today().isoformat()
 
@@ -292,3 +322,39 @@ class DanbooruPopularExtractor(DanbooruExtractor):
 
     def posts(self):
         return self._pagination("/explore/posts/popular.json", self.params)
+
+
+class DanbooruArtistExtractor(DanbooruExtractor):
+    """Extractor for danbooru artists"""
+    subcategory = "artist"
+    pattern = BASE_PATTERN + r"/artists/(\d+)"
+    example = "https://danbooru.donmai.us/artists/12345"
+
+    items = DanbooruExtractor.items_artists
+
+    def artists(self):
+        url = "{}/artists/{}.json".format(self.root, self.groups[-1])
+        return (self.request(url).json(),)
+
+
+class DanbooruArtistSearchExtractor(DanbooruExtractor):
+    """Extractor for danbooru artist searches"""
+    subcategory = "artist-search"
+    pattern = BASE_PATTERN + r"/artists/?\?([^#]+)"
+    example = "https://danbooru.donmai.us/artists?QUERY"
+
+    items = DanbooruExtractor.items_artists
+
+    def artists(self):
+        url = self.root + "/artists.json"
+        params = text.parse_query(self.groups[-1])
+        params["page"] = text.parse_int(params.get("page"), 1)
+
+        while True:
+            artists = self.request(url, params=params).json()
+
+            yield from artists
+
+            if len(artists) < 20:
+                return
+            params["page"] += 1
