@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 
-# Copyright 2022 Mike Fährmann
+# Copyright 2022-2023 Mike Fährmann
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License version 2 as
@@ -24,10 +24,8 @@ class ItakuExtractor(Extractor):
     archive_fmt = "{id}"
     request_interval = (0.5, 1.5)
 
-    def __init__(self, match):
-        Extractor.__init__(self, match)
+    def _init(self):
         self.api = ItakuAPI(self)
-        self.item = match.group(1)
         self.videos = self.config("videos", True)
 
     def items(self):
@@ -60,79 +58,30 @@ class ItakuExtractor(Extractor):
 class ItakuGalleryExtractor(ItakuExtractor):
     """Extractor for posts from an itaku user gallery"""
     subcategory = "gallery"
-    pattern = BASE_PATTERN + r"/profile/([^/?#]+)/gallery"
-    test = ("https://itaku.ee/profile/piku/gallery", {
-        "pattern": r"https://d1wmr8tlk3viaj\.cloudfront\.net/gallery_imgs"
-                   r"/[^/?#]+\.(jpg|png|gif)",
-        "range": "1-10",
-        "count": 10,
-    })
+    pattern = BASE_PATTERN + r"/profile/([^/?#]+)/gallery(?:/(\d+))?"
+    example = "https://itaku.ee/profile/USER/gallery"
 
     def posts(self):
-        return self.api.galleries_images(self.item)
+        return self.api.galleries_images(*self.groups)
 
 
 class ItakuImageExtractor(ItakuExtractor):
     subcategory = "image"
     pattern = BASE_PATTERN + r"/images/(\d+)"
-    test = (
-        ("https://itaku.ee/images/100471", {
-            "pattern": r"https://d1wmr8tlk3viaj\.cloudfront\.net/gallery_imgs"
-                       r"/220504_oUNIAFT\.png",
-            "count": 1,
-            "keyword": {
-                "already_pinned": None,
-                "blacklisted": {
-                    "blacklisted_tags": [],
-                    "is_blacklisted": False
-                },
-                "can_reshare": True,
-                "date": "dt:2022-05-05 19:21:17",
-                "date_added": "2022-05-05T19:21:17.674148Z",
-                "date_edited": "2022-05-25T14:37:46.220612Z",
-                "description": "sketch from drawpile",
-                "extension": "png",
-                "filename": "220504_oUNIAFT",
-                "hotness_score": float,
-                "id": 100471,
-                "image": "https://d1wmr8tlk3viaj.cloudfront.net/gallery_imgs"
-                         "/220504_oUNIAFT.png",
-                "image_xl": "https://d1wmr8tlk3viaj.cloudfront.net"
-                            "/gallery_imgs/220504_oUNIAFT/xl.jpg",
-                "liked_by_you": False,
-                "maturity_rating": "SFW",
-                "num_comments": 2,
-                "num_likes": 80,
-                "num_reshares": 2,
-                "obj_tags": 136446,
-                "owner": 16775,
-                "owner_avatar": "https://d1wmr8tlk3viaj.cloudfront.net"
-                                "/profile_pics/av2022r_vKYVywc/sm.jpg",
-                "owner_displayname": "Piku",
-                "owner_username": "piku",
-                "reshared_by_you": False,
-                "sections": ["Fanart/Miku"],
-                "tags": list,
-                "tags_character": ["hatsune_miku"],
-                "tags_copyright": ["vocaloid"],
-                "tags_general"  : ["twintails", "green_hair", "flag", "gloves",
-                                   "green_eyes", "female", "racing_miku"],
-                "title": "Racing Miku 2022 Ver.",
-                "too_mature": False,
-                "uncompressed_filesize": "0.62",
-                "video": None,
-                "visibility": "PUBLIC",
-            },
-        }),
-        # video
-        ("https://itaku.ee/images/19465", {
-            "pattern": r"https://d1wmr8tlk3viaj\.cloudfront\.net/gallery_vids"
-                       r"/sleepy_af_OY5GHWw\.mp4",
-        }),
-    )
+    example = "https://itaku.ee/images/12345"
 
     def posts(self):
-        return (self.api.image(self.item),)
+        return (self.api.image(self.groups[0]),)
+
+
+class ItakuSearchExtractor(ItakuExtractor):
+    subcategory = "search"
+    pattern = BASE_PATTERN + r"/home/images/?\?([^#]+)"
+    example = "https://itaku.ee/home/images?tags=SEARCH"
+
+    def posts(self):
+        params = text.parse_query_list(self.groups[0])
+        return self.api.search_images(params)
 
 
 class ItakuAPI():
@@ -142,17 +91,52 @@ class ItakuAPI():
         self.root = extractor.root + "/api"
         self.headers = {
             "Accept": "application/json, text/plain, */*",
-            "Referer": extractor.root + "/",
         }
 
+    def search_images(self, params):
+        endpoint = "/galleries/images/"
+        required_tags = []
+        negative_tags = []
+        optional_tags = []
+
+        tags = params.pop("tags", None)
+        if not tags:
+            tags = ()
+        elif isinstance(tags, str):
+            tags = (tags,)
+
+        for tag in tags:
+            if not tag:
+                pass
+            elif tag[0] == "-":
+                negative_tags.append(tag[1:])
+            elif tag[0] == "~":
+                optional_tags.append(tag[1:])
+            else:
+                required_tags.append(tag)
+
+        api_params = {
+            "required_tags": required_tags,
+            "negative_tags": negative_tags,
+            "optional_tags": optional_tags,
+            "date_range": "",
+            "maturity_rating": ("SFW", "Questionable", "NSFW"),
+            "ordering"  : "-date_added",
+            "page"      : "1",
+            "page_size" : "30",
+            "visibility": ("PUBLIC", "PROFILE_ONLY"),
+        }
+        api_params.update(params)
+        return self._pagination(endpoint, api_params, self.image)
+
     def galleries_images(self, username, section=None):
         endpoint = "/galleries/images/"
         params = {
             "cursor"    : None,
             "owner"     : self.user(username)["owner"],
-            "section"   : section,
+            "sections"  : section,
             "date_range": "",
-            "maturity_rating": ("SFW", "Questionable", "NSFW", "Extreme"),
+            "maturity_rating": ("SFW", "Questionable", "NSFW"),
             "ordering"  : "-date_added",
             "page"      : "1",
             "page_size" : "30",
