@@ -27,8 +27,10 @@ class PixivExtractor(Extractor):
     filename_fmt = "{id}_p{num}.{extension}"
     archive_fmt = "{id}{suffix}.{extension}"
     cookies_domain = ".pixiv.net"
-    sanity_url = "https://s.pximg.net/common/images/limit_sanity_level_360.png"
-    mypixiv_url = "https://s.pximg.net/common/images/limit_mypixiv_360.png"
+    limit_url = "https://s.pximg.net/common/images/limit_"
+    # https://s.pximg.net/common/images/limit_sanity_level_360.png
+    # https://s.pximg.net/common/images/limit_unviewable_360.png
+    # https://s.pximg.net/common/images/limit_mypixiv_360.png
 
     def _init(self):
         self.api = PixivAppAPI(self)
@@ -69,9 +71,12 @@ class PixivExtractor(Extractor):
             if self.meta_user:
                 work.update(self.api.user_detail(work["user"]["id"]))
             if self.meta_comments:
-                if work["total_comments"]:
-                    work["comments"] = list(
-                        self.api.illust_comments(work["id"]))
+                if work["total_comments"] and not work.get("_ajax"):
+                    try:
+                        work["comments"] = list(
+                            self.api.illust_comments(work["id"]))
+                    except Exception:
+                        work["comments"] = ()
                 else:
                     work["comments"] = ()
             if self.meta_bookmark and work["is_bookmarked"]:
@@ -111,24 +116,39 @@ class PixivExtractor(Extractor):
                 {
                     "url"   : img["image_urls"]["original"],
                     "suffix": "_p{:02}".format(num),
+                    "_fallback": self._fallback_image(img),
                 }
                 for num, img in enumerate(meta_pages)
             ]
 
         url = meta_single_page["original_image_url"]
-        if url == self.sanity_url:
-            work["_ajax"] = True
-            self.log.warning("%s: 'limit_sanity_level' warning", work["id"])
-            if self.sanity_workaround:
-                body = self._request_ajax("/illust/" + str(work["id"]))
-                return self._extract_ajax(work, body)
+        if url.startswith(self.limit_url):
+            work_id = work["id"]
+            self.log.debug("%s: %s", work_id, url)
 
-        elif url == self.mypixiv_url:
-            work["_mypixiv"] = True
-            self.log.warning("%s: 'My pixiv' locked", work["id"])
+            limit_type = url.rpartition("/")[2]
+            if limit_type in (
+                "limit_",  # for '_extend_sanity()' inserts
+                "limit_unviewable_360.png",
+                "limit_sanity_level_360.png",
+            ):
+                work["_ajax"] = True
+                self.log.warning("%s: 'limit_sanity_level' warning", work_id)
+                if self.sanity_workaround:
+                    body = self._request_ajax("/illust/" + str(work_id))
+                    return self._extract_ajax(work, body)
+
+            elif limit_type == "limit_mypixiv_360.png":
+                work["_mypixiv"] = True
+                self.log.warning("%s: 'My pixiv' locked", work_id)
+
+            else:
+                work["_mypixiv"] = True  # stop further processing
+                self.log.error("%s: Unknown 'limit' URL type: %s",
+                               work_id, limit_type)
 
         elif work["type"] != "ugoira":
-            return ({"url": url},)
+            return ({"url": url, "_fallback": self._fallback_image(url)},)
 
         elif self.load_ugoira:
             try:
@@ -269,6 +289,24 @@ class PixivExtractor(Extractor):
             except exception.HttpError:
                 pass
 
+    def _fallback_image(self, src):
+        if isinstance(src, str):
+            urls = None
+            orig = src
+        else:
+            urls = src["image_urls"]
+            orig = urls["original"]
+
+        base = orig.rpartition(".")[0]
+        yield base.replace("-original/", "-master/", 1) + "_master1200.jpg"
+
+        if urls is None:
+            return
+
+        for fmt in ("large", "medium", "square_medium"):
+            if fmt in urls:
+                yield urls[fmt]
+
     @staticmethod
     def _date_from_url(url, offset=timedelta(hours=9)):
         try:
@@ -411,7 +449,7 @@ class PixivArtworksExtractor(PixivExtractor):
                 elif ajax_id > work_id:
                     index -= 1
                     self.log.debug("Inserting work %s", ajax_id)
-                    yield self._make_work(ajax_id, self.sanity_url, user)
+                    yield self._make_work(ajax_id, self.limit_url, user)
 
                 else:  # ajax_id < work_id
                     break
@@ -421,7 +459,7 @@ class PixivArtworksExtractor(PixivExtractor):
         while index >= 0:
             ajax_id = ajax_ids[index]
             self.log.debug("Inserting work %s", ajax_id)
-            yield self._make_work(ajax_id, self.sanity_url, user)
+            yield self._make_work(ajax_id, self.limit_url, user)
             index -= 1
 
 
