@@ -241,9 +241,10 @@ class SankakuAPI():
 
             if response.status_code == 429:
                 until = response.headers.get("X-RateLimit-Reset")
-                if not until and b"tags-limit" in response.content:
-                    raise exception.StopExtraction("Search tag limit exceeded")
-                seconds = None if until else 60
+                if not until and b"_tags-explicit-limit" in response.content:
+                    raise exception.AuthorizationError(
+                        "Search tag limit exceeded")
+                seconds = None if until else 600
                 self.extractor.wait(until=until, seconds=seconds)
                 continue
 
