@@ -9,7 +9,9 @@
 """Collection of functions that work on strings/text"""
 
 import re
+import sys
 import html
+import time
 import datetime
 import urllib.parse
 
@@ -59,15 +61,21 @@ def ensure_http_scheme(url, scheme="https://"):
 def root_from_url(url, scheme="https://"):
     """Extract scheme and domain from a URL"""
     if not url.startswith(("https://", "http://")):
-        return scheme + url[:url.index("/")]
-    return url[:url.index("/", 8)]
+        try:
+            return scheme + url[:url.index("/")]
+        except ValueError:
+            return scheme + url
+    try:
+        return url[:url.index("/", 8)]
+    except ValueError:
+        return url
 
 
 def filename_from_url(url):
     """Extract the last part of an URL to use as a filename"""
     try:
         return url.partition("?")[0].rpartition("/")[2]
-    except (TypeError, AttributeError):
+    except Exception:
         return ""
 
 
@@ -116,17 +124,26 @@ def extract(txt, begin, end, pos=0):
         first = txt.index(begin, pos) + len(begin)
         last = txt.index(end, first)
         return txt[first:last], last+len(end)
-    except (ValueError, TypeError, AttributeError):
+    except Exception:
         return None, pos
 
 
+def extr(txt, begin, end, default=""):
+    """Stripped-down version of 'extract()'"""
+    try:
+        first = txt.index(begin) + len(begin)
+        return txt[first:txt.index(end, first)]
+    except Exception:
+        return default
+
+
 def rextract(txt, begin, end, pos=-1):
     try:
         lbeg = len(begin)
         first = txt.rindex(begin, 0, pos)
         last = txt.index(end, first + lbeg)
         return txt[first + lbeg:last], first
-    except (ValueError, TypeError, AttributeError):
+    except Exception:
         return None, pos
 
 
@@ -143,16 +160,16 @@ def extract_all(txt, rules, pos=0, values=None):
 
 def extract_iter(txt, begin, end, pos=0):
     """Yield values that would be returned by repeated calls of extract()"""
-    index = txt.index
-    lbeg = len(begin)
-    lend = len(end)
     try:
+        index = txt.index
+        lbeg = len(begin)
+        lend = len(end)
         while True:
             first = index(begin, pos) + lbeg
             last = index(end, first)
             pos = last + lend
             yield txt[first:last]
-    except (ValueError, TypeError, AttributeError):
+    except Exception:
         return
 
 
@@ -165,7 +182,7 @@ def extract_from(txt, pos=0, default=""):
             last = index(end, first)
             pos = last + len(end)
             return txt[first:last]
-        except (ValueError, TypeError, AttributeError):
+        except Exception:
             return default
     return extr
 
@@ -185,7 +202,7 @@ def parse_bytes(value, default=0, suffixes="bkmgtp"):
     """Convert a bytes-amount ("500k", "2.5M", ...) to int"""
     try:
         last = value[-1].lower()
-    except (TypeError, LookupError):
+    except Exception:
         return default
 
     if last in suffixes:
@@ -206,7 +223,7 @@ def parse_int(value, default=0):
         return default
     try:
         return int(value)
-    except (ValueError, TypeError):
+    except Exception:
         return default
 
 
@@ -216,28 +233,76 @@ def parse_float(value, default=0.0):
         return default
     try:
         return float(value)
-    except (ValueError, TypeError):
+    except Exception:
         return default
 
 
 def parse_query(qs):
-    """Parse a query string into key-value pairs"""
+    """Parse a query string into name-value pairs
+
+    Ignore values whose name has been seen before
+    """
+    if not qs:
+        return {}
+
     result = {}
     try:
-        for key, value in urllib.parse.parse_qsl(qs):
-            if key not in result:
-                result[key] = value
-    except AttributeError:
+        for name_value in qs.split("&"):
+            name, eq, value = name_value.partition("=")
+            if eq:
+                name = unquote(name.replace("+", " "))
+                if name not in result:
+                    result[name] = unquote(value.replace("+", " "))
+    except Exception:
         pass
     return result
 
 
-def parse_timestamp(ts, default=None):
-    """Create a datetime object from a unix timestamp"""
+def parse_query_list(qs):
+    """Parse a query string into name-value pairs
+
+    Combine values of duplicate names into lists
+    """
+    if not qs:
+        return {}
+
+    result = {}
     try:
-        return datetime.datetime.utcfromtimestamp(int(ts))
-    except (TypeError, ValueError, OverflowError):
-        return default
+        for name_value in qs.split("&"):
+            name, eq, value = name_value.partition("=")
+            if eq:
+                name = unquote(name.replace("+", " "))
+                value = unquote(value.replace("+", " "))
+                if name in result:
+                    rvalue = result[name]
+                    if isinstance(rvalue, list):
+                        rvalue.append(value)
+                    else:
+                        result[name] = [rvalue, value]
+                else:
+                    result[name] = value
+    except Exception:
+        pass
+    return result
+
+
+if sys.hexversion < 0x30c0000:
+    # Python <= 3.11
+    def parse_timestamp(ts, default=None):
+        """Create a datetime object from a Unix timestamp"""
+        try:
+            return datetime.datetime.utcfromtimestamp(int(ts))
+        except Exception:
+            return default
+else:
+    # Python >= 3.12
+    def parse_timestamp(ts, default=None):
+        """Create a datetime object from a Unix timestamp"""
+        try:
+            Y, m, d, H, M, S, _, _, _ = time.gmtime(int(ts))
+            return datetime.datetime(Y, m, d, H, M, S)
+        except Exception:
+            return default
 
 
 def parse_datetime(date_string, format="%Y-%m-%dT%H:%M:%S%z", utcoffset=0):
